import numpy as np


def validate_points(points):
    # If the user is tracking only a single point, reformat it slightly.
    if points.shape == (2,):
        points = points[np.newaxis, ...]
    else:
        if points.shape[1] != 2 or len(points.shape) > 2:
            print(f"The shape of `Detection.points` should be (num_of_points_to_track, 2), not {points.shape}.")
            print("Check your detection conversion code.")
            exit()
    return points



import cv2
import numpy as np
from .utils import validate_points

def draw_points(frame, detections, radius=None, thickness=None, color=None):
    if detections is None:
        return
    frame_scale = frame.shape[0] / 100
    if radius is None:
        radius = int(max(frame_scale * 0.7, 1))
    if thickness is None:
        thickness = int(max(frame_scale / 7, 1))
    if color is None:
        color = Color.red

    for d in detections:
        points = d.points
        points = validate_points(points)

        for point in points:
            cv2.circle(frame, tuple(point.astype(int)), radius=radius, color=color, thickness=thickness)

def draw_tracked_objects(frame, objects, radius=None, color=None, id_size=None, id_thickness=None, draw_points=True):
    frame_scale = frame.shape[0] / 100
    if radius is None:
        radius = int(frame_scale * 0.5)
    if id_size is None:
        id_size = frame_scale / 10
    if id_thickness is None:
        id_thickness = int(frame_scale / 5)

    for obj in objects:
        if color is None:
            point_color = Color.random(obj.id)
            id_color = point_color
        else:
            point_color = color
            id_color = color

        if draw_points:
            for point in obj.estimate:
                cv2.circle(frame, tuple(point.astype(int)), radius=radius, color=point_color, thickness=-1)

        if id_size > 0:
            id_draw_position = centroid(obj.estimate)
            cv2.putText(
                frame, str(obj.id), id_draw_position, cv2.FONT_HERSHEY_SIMPLEX, id_size,
                id_color, id_thickness, cv2.LINE_AA
            )

def draw_debug_metrics(frame, objects, text_size=None, text_thickness=None, color=None,
                       only_ids=None, only_initializing_ids=None):
    """Draw objects with their debug information

    It is recommended to set the input variable `objects` to `your_tracker_object.objects`
    so you can also debug objects wich haven't finished initializing, and you get a more
    complete view of what your tracker is doing on each step.
    """
    frame_scale = frame.shape[0] / 100
    if text_size is None:
        text_size = frame_scale / 10
    if text_thickness is None:
        text_thickness = int(frame_scale / 5)
    radius = int(frame_scale * 0.5)

    for obj in objects:
        if only_ids is not None:
            if obj.id not in only_ids: continue
        if only_initializing_ids is not None:
            if obj.initializing_id not in only_initializing_ids: continue
        if color is None:
            text_color = Color.random(obj.initializing_id)
        else:
            text_color = color
        draw_position = centroid(obj.estimate)

        for point in obj.estimate:
            cv2.circle(frame, tuple(point.astype(int)), radius=radius, color=text_color, thickness=-1)

        # Interframe distance
        dist = obj.last_distance
        if dist is None:
            dist = "-"
        elif dist > 1000:
            dist = ">"
        else:
            dist = "{:.1f}".format(dist)

        # No support for multiline text in opencv :facepalm:
        lines_to_draw = (
            "{}|{}".format(obj.id, obj.initializing_id),
            "d:{}".format(dist),
            "a:{}".format(obj.age),
            "h:{}".format(obj.hit_counter),
        )
        for i, line in enumerate(lines_to_draw):
            draw_position = (
                int(draw_position[0]),
                int(draw_position[1] + i * text_size * 7 + 15)
            )
            cv2.putText(
                frame, line, draw_position, cv2.FONT_HERSHEY_SIMPLEX, text_size,
                text_color, text_thickness, cv2.LINE_AA
            )

def centroid(points):
    length = points.shape[0]
    sum_x = np.sum(points[:, 0])
    sum_y = np.sum(points[:, 1])
    return int(sum_x/length), int(sum_y/length)

class Color():
    green = (0, 128, 0)
    white = (255, 255, 255)
    olive = (0, 128, 128)
    black = (0, 0, 0)
    navy = (128, 0, 0)
    red = (0, 0, 255)
    maroon = (0, 0, 128)
    grey = (128, 128, 128)
    purple = (128, 0, 128)
    yellow = (0, 255, 255)
    lime = (0, 255, 0)
    fuchsia = (255, 0, 255)
    aqua = (255, 255, 0)
    blue = (255, 0, 0)
    teal = (128, 128, 0)
    silver = (192, 192, 192)

    @staticmethod
    def random(obj_id):
        color_list = [
            c for c in Color.__dict__.keys()
            if c[:2] != "__" and c not in ("random", "red", "white", "grey", "black")
        ]
        return getattr(Color, color_list[obj_id % len(color_list)])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['norfair']

package_data = \
{'': ['*']}

install_requires = \
['filterpy>=1.4.5,<2.0.0', 'opencv-python>=4.2.0,<5.0.0', 'rich>=5.0.0,<6.0.0']

setup_kwargs = {
    'name': 'norfair',
    'version': '0.1.3',
    'description': 'Simple object tracker',
    'long_description': '# Norfair\n',
    'author': 'joaqo',
    'author_email': 'joaquin.alori@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tryolabs/norfair',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import cv2
import os
import time

from rich import print
from rich.progress import Progress, BarColumn, TimeRemainingColumn

class Video():

    def __init__(self, input_path, output_path="."):
        self.input_path = input_path
        self.output_path = output_path
        self.output_video = None

        # Read Input Video
        self.video_capture = cv2.VideoCapture(self.input_path)
        total_frames = int(self.video_capture.get(cv2.CAP_PROP_FRAME_COUNT))
        if total_frames == 0:
            print("[bold red]Error reading input video file:[/bold red] Make sure file exists and is a video file.")
            if "~" in self.input_path:
                print("Using ~ as abbreviation for your home folder is not supported.")
            exit()
        self.frame_height = int(self.video_capture.get(cv2.CAP_PROP_FRAME_HEIGHT))
        self.frame_width = int(self.video_capture.get(cv2.CAP_PROP_FRAME_WIDTH))
        self.fps = self.video_capture.get(cv2.CAP_PROP_FPS)
        self.frame_counter = 0

        # Setup progressbar
        file_name = os.path.basename(self.input_path)
        _, terminal_columns = os.popen('stty size', 'r').read().split()
        space_for_filename = int(terminal_columns) - 40  # Leave 40 space for progressbar
        abbreviated_file_name = (
            file_name if len(file_name) < space_for_filename
            else "{} ... {}".format(file_name[:space_for_filename // 2 - 3],
                                    file_name[-space_for_filename // 2 + 3:])
        )
        label = "{} {}x{}@{:.0f}fps".format(
            abbreviated_file_name, self.frame_width, self.frame_height, self.fps
        )
        self.progress_bar = Progress(
            "[progress.description]{task.description}",
            BarColumn(),
            "[progress.percentage]{task.percentage:>3.0f}%",
            TimeRemainingColumn(),
            "[yellow]{task.fields[fps]:.2f}fps",
            auto_refresh=False,
            redirect_stdout=False,
            redirect_stderr=False
        )
        self.task = self.progress_bar.add_task(label, total=total_frames, fps=0)

    # This is a generator, note the yield keyword below.
    def __iter__(self):
        with self.progress_bar as progress_bar:
            start = time.time()

            # Iterate over video
            while True:
                self.frame_counter += 1
                ret, frame = self.video_capture.read()
                if ret is False or frame is None:
                    break
                process_fps = self.frame_counter / (time.time() - start)
                progress_bar.update(self.task, advance=1, refresh=True, fps=process_fps)
                yield frame

        # Cleanup
        if self.output_video is not None:
            self.output_video.release()
        self.video_capture.release()
        cv2.destroyAllWindows()

    def write(self, frame):
        if self.output_video is None:
            # The user may need to access the output file path on their code
            self.output_file_path = self.generate_output_path('.avi')
            fourcc = cv2.VideoWriter_fourcc(*'XVID')
            self.output_video = cv2.VideoWriter(
                # TODO: Can we do mp4 instead of avi? Make this configurable?
                self.output_file_path,
                fourcc,
                self.fps,
                (self.frame_width, self.frame_height)
            )
        self.output_video.write(frame)
        cv2.waitKey(1)

    def show(self, frame, downsample_ratio=1):
        # Resize to lower resolution for faster streaming over slow connections
        if downsample_ratio is not None:
            frame = cv2.resize(
                frame,
                (int(self.frame_width / downsample_ratio),
                 int(self.frame_height / downsample_ratio))
            )
        cv2.imshow("Output", frame)
        cv2.waitKey(1)

    def generate_output_path(self, extension, prefix=''):
        if os.path.isdir(self.output_path):
            base_file_name = self.input_path.split('/')[-1].split('.')[0]
            file_name = prefix + base_file_name + "_out" + extension
            return os.path.join(self.output_path, file_name)
        elif self.output_path[-4:] in [".avi", ".mp4"]:
            return self.output_path
        else:
            # TODO: Check that we support mp4
            print(f"'{self.output_path}' is not an existing folder or an avi/mp4 filename.")
            print("Exiting...")
            exit()
